 /* stod - convert Fortran program from double-precision to single-precision */
%{
#include <ctype.h>
#undef output				/* so we get our private version */
void	convert_declaration();
void	convert_number();
void	cvtstr();
void	output();
void	outstr();
#undef ECHO
#define ECHO	outstr(yytext)
%}
		/* increase two lex table sizes for Sun lex */
		/* (IBM RS/6000 AIX lex does not need this) */
%a 3000
%o 7000

a	[aA]
b	[bB]
c	[cC]
d	[dD]
e	[eE]
f	[fF]
g	[gG]
h	[hH]
i	[iI]
j	[jJ]
k	[kK]
l	[lL]
m	[mM]
n	[nN]
o	[oO]
p	[pP]
q	[qQ]
r	[rR]
s	[sS]
t	[tT]
u	[uU]
v	[vV]
w	[wW]
x	[xX]
y	[yY]
z	[zZ]
					/* decimal digit */
D		[0-9]
					/* floating-point exponent */
E		[DEde][-+]?{D}+
					/* optional whitespace */
W		[ \t]*
					/* number */
N		{D}+
%%
^[*Cc].*$			ECHO;	/* copy comments unchanged */

^\t[1-9]			{	/* handle tabbed continuation */
				    outstr("     ");
				    output(yytext[1]);
				}

^\t0 |
^\t				outstr("      "); /* handle tabbed initial */

^{W}{r}{e}{a}{l}"*"8		convert_declaration();

^{W}{r}{e}{a}{l}"*"4 |
^{W}{r}{e}{a}{l}/[^*a-zA-Z]	convert_declaration();

{d}{o}{u}{b}{l}{e}{W}{p}{r}{e}{c}{i}{s}{i}{o}{n}  convert_declaration();

^{W}{i}{m}{p}{l}{i}{c}{i}{t}{W}{r}{e}{a}{l}"*"8		convert_declaration();

^{W}{i}{m}{p}{l}{i}{c}{i}{t}{W}{r}{e}{a}{l}"*"4 |
^{W}{i}{m}{p}{l}{i}{c}{i}{t}{W}{r}{e}{a}{l}/[^*a-zA-Z]	convert_declaration();

^{W}{i}{m}{p}{l}{i}{c}{i}{t}{W}{d}{o}{u}{b}{l}{e}{W}{p}{r}{e}{c}{i}{s}{i}{o}{n}  convert_declaration();

^"     "[^ 0]			ECHO;	/* copy continuation line prefixes */

{N}{p}{D}*[A-Za-z]{N}"."{N}	ECHO;	/* copy FORMAT items intact */

{D}*[A-Za-z]{N}"."{N}		ECHO;	/* copy FORMAT items intact */

"."{n}{o}{t}"." |
"."{a}{n}{d}"." |
"."{x}{o}{r}"." |
"."{o}{r}"." |
"."{g}{e}"." |
"."{g}{t}"." |
"."{l}{e}"." |
"."{l}{t}"." |
"."{e}{q}"." |
"."{e}{q}{v}"." |
"."{n}{e}"." |
"."{n}{e}{q}"."			ECHO;	/* recognize logical operators */

{N}"."{D}*({E})? |
{D}*"."{N}({E})? |
{N}{E}				convert_number();

{d}{a}{b}{s}			cvtstr("abs");	 /* handle name conversions */
{d}{a}{c}{o}{s}			cvtstr("acos");
{d}{a}{s}{i}{n}			cvtstr("asin");
{d}{a}{t}{a}{n}			cvtstr("atan");
{d}{a}{t}{a}{n}2		cvtstr("atan2");
{d}{b}{l}{e}			cvtstr("sngl");
{d}{c}{o}{s}			cvtstr("cos");
{d}{c}{o}{s}{h}			cvtstr("cosh");
{d}{d}{i}{m}			cvtstr("dim");
{d}{e}{x}{p}			cvtstr("exp");
{d}{f}{l}{o}{a}{t}		cvtstr("float");
{d}{i}{n}{t}			cvtstr("aint");
{d}{l}{o}{g}			cvtstr("alog");
{d}{l}{o}{g}10			cvtstr("alog10");
{d}{m}{a}{x}0			cvtstr("amax0");
{d}{m}{a}{x}1			cvtstr("amax1");
{d}{m}{i}{n}0			cvtstr("amin0");
{d}{m}{i}{n}1			cvtstr("amin1");
{d}{m}{o}{d}			cvtstr("amod");
{d}{n}{i}{n}{t}			cvtstr("anint");
{d}{s}{i}{g}{n}			cvtstr("sign");
{d}{s}{i}{n}			cvtstr("sin");
{d}{s}{i}{n}{h}			cvtstr("sinh");
{d}{s}{q}{r}{t}			cvtstr("sqrt");
{d}{t}{a}{n}			cvtstr("tan");
{d}{t}{a}{n}{h}			cvtstr("tanh");
{i}{d}{i}{n}{t}			cvtstr("int");
{i}{d}{n}{i}{n}{t}		cvtstr("nint");

[A-Za-z][A-Za-z0-9]+ |
[0-9]+ |
\n |
.				ECHO;
%%
void
convert_declaration()
{
    char *p;

    for (p = (char*)&yytext[0]; *p && isspace(*p); ++p)
	output(*p);
    if (*p == 'i')
	outstr("implicit ");
    else if (*p == 'I')
	outstr("IMPLICIT ");
    outstr(islower(*p) ? "real" : "REAL");
}

void
convert_number()
{
    char *p;
    int exponent_seen = 0;

    for (p = (char*)&yytext[0]; *p; ++p)
    {
	if ((*p == 'D') || (*p == 'd'))
	{
	    exponent_seen++;
	    *p += 'e' - 'd';
	}
	else if ((*p == 'E') || (*p == 'e'))
	    exponent_seen++;
    }
    ECHO;
    if (!exponent_seen)
	outstr("E+00",stdout);
}

void
cvtstr(s)			/* output possibly case-converted string */
char *s;
{
    if (islower(yytext[0]))
	outstr(s);
    else
    {
	for ( ; *s ; ++s)
	    output(islower(*s) ? toupper(*s) : *s);
    }
}

void
output(c)		/* private output() routine to handle line wrapping */
int c;
{
    static int    last_column = 0;

    switch (c)
    {
    case '\n':
	last_column = 0;
	break;

    default:
	last_column++;
	break;
    }
    if ((last_column > 72) && (c != '\n'))
    {
	printf("\n     X");		/* all output happens here */
	last_column = 7;
    }
    putchar(c);				/* and here only */
}

void
outstr(s)				/* output string with line wrapping */
char *s;
{
    for ( ; *s ; ++s)
	output(*s);
}
