 /* stod - convert Fortran program from single-precision to double-precision */
%{
#include <ctype.h>
#undef output				/* so we get our private version */
void	convert_declaration();
void	convert_number();
void	cvtstr();
void	output();
void	outstr();
#undef ECHO
#define ECHO	outstr(yytext)
%}
					/* increase two lex table sizes */
%a 4000
%o 8000

a	[aA]
b	[bB]
c	[cC]
d	[dD]
e	[eE]
f	[fF]
g	[gG]
h	[hH]
i	[iI]
j	[jJ]
k	[kK]
l	[lL]
m	[mM]
n	[nN]
o	[oO]
p	[pP]
q	[qQ]
r	[rR]
s	[sS]
t	[tT]
u	[uU]
v	[vV]
w	[wW]
x	[xX]
y	[yY]
z	[zZ]

				    /* digits with optional embedded blanks */
B		([0-9]|[0-9][0-9 ]*[0-9])
					/* decimal digit */
D		[0-9]
					/* floating-point exponent */
E		[DEde][-+]?{B}
					/* optional whitespace */
W		[ \t]*
					/* number */
N		{B}
%%
^[*Cc].*$			ECHO;	/* copy comments unchanged */

^\t[1-9]			{	/* handle tabbed continuation */
				    outstr("     ");
				    output(yytext[1]);
				}

^\t0 |
^\t				outstr("      "); /* handle tabbed initial */

^{W}{r}{e}{a}{l}"*"8		convert_declaration();

^{W}{r}{e}{a}{l}"*"4 |
^{W}{r}{e}{a}{l}/[^*a-zA-Z]	convert_declaration();

^{W}{i}{m}{p}{l}{i}{c}{i}{t}{W}{r}{e}{a}{l}"*"8		convert_declaration();

^{W}{i}{m}{p}{l}{i}{c}{i}{t}{W}{r}{e}{a}{l}"*"4 |
^{W}{i}{m}{p}{l}{i}{c}{i}{t}{W}{r}{e}{a}{l}/[^*a-zA-Z]	convert_declaration();

^"     "[^ 0]			ECHO;	/* copy continuation line prefixes */

{N}{p}{N}*[A-Za-z]{N}"."{N}	ECHO;	/* copy FORMAT items intact */

{N}*[A-Za-z]{N}"."{N}		ECHO;	/* copy FORMAT items intact */

"."{n}{o}{t}"." |
"."{a}{n}{d}"." |
"."{x}{o}{r}"." |
"."{o}{r}"." |
"."{g}{e}"." |
"."{g}{t}"." |
"."{l}{e}"." |
"."{l}{t}"." |
"."{e}{q}"." |
"."{e}{q}{v}"." |
"."{n}{e}"." |
"."{n}{e}{q}"."			ECHO;	/* recognize logical operators */

{N}"."{N}*({W}{E})? |
{N}*"."{N}({W}{E})? |
{N}{W}{E}			convert_number() ;

{i}{f}{i}{x}			cvtstr("idint"); /* handle name conversions */
{i}{n}{t}			cvtstr("idint");
{n}{i}{n}{t}			cvtstr("idnint");
{s}{n}{g}{l}			cvtstr("dble");
{a}{i}{n}{t}			cvtstr("dint");
{a}{l}{o}{g}			cvtstr("dlog");
{a}{l}{o}{g}10			cvtstr("dlog10");
{a}{m}{a}{x}0			cvtstr("dmax0");
{a}{m}{a}{x}1			cvtstr("dmax1");
{a}{m}{i}{n}0			cvtstr("dmin0");
{a}{m}{i}{n}1			cvtstr("dmin1");
{a}{m}{o}{d}			cvtstr("dmod");
{a}{n}{i}{n}{t}			cvtstr("dnint");
{a}{b}{s}			cvtstr("dabs");
{a}{c}{o}{s}			cvtstr("dacos");
{a}{s}{i}{n}			cvtstr("dasin");
{a}{t}{a}{n}			cvtstr("datan");
{a}{t}{a}{n}2			cvtstr("datan2");
{c}{o}{s}			cvtstr("dcos");
{c}{o}{s}{h}			cvtstr("dcosh");
{d}{i}{m}			cvtstr("ddim");
{e}{x}{p}			cvtstr("dexp");
{f}{l}{o}{a}{t}			cvtstr("dfloat");
{r}{a}{n}{d}			cvtstr("drand");
{s}{i}{g}{n}			cvtstr("dsign");
{s}{i}{n}			cvtstr("dsin");
{s}{i}{n}{h}			cvtstr("dsinh");
{s}{q}{r}{t}			cvtstr("dsqrt");
{t}{a}{n}			cvtstr("dtan");
{t}{a}{n}{h}			cvtstr("dtanh");

[A-Za-z][A-Za-z0-9]+ |
[0-9]+ |
\n |
.				ECHO;
%%
void
convert_declaration()
{
    char *p;

    for (p = yytext; *p && isspace(*p); ++p)
	output(*p);
    if (*p == 'i')
	outstr("implicit ");
    else if (*p == 'I')
	outstr("IMPLICIT ");
    outstr(islower(*p) ? "double precision" : "DOUBLE PRECISION");
}

void
convert_number()
{
    char *p;
    int exponent_seen = 0;

    for (p = yytext; *p; ++p)
    {
	if ((*p == 'E') || (*p == 'e'))
	{
	    exponent_seen++;
	    *p += 'd' - 'e';
	}
	else if ((*p == 'D') || (*p == 'd'))
	    exponent_seen++;
    }
    ECHO;
    if (!exponent_seen)
	outstr("D+00");
}

void
cvtstr(s)			/* output possibly case-converted string */
char *s;
{
    if (islower(yytext[0]))
	outstr(s);
    else
    {
	for ( ; *s ; ++s)
	    output(islower(*s) ? toupper(*s) : *s);
    }
}

void
output(c)		/* private output() routine to handle line wrapping */
int c;
{
    static int    last_column = 0;

    switch (c)
    {
    case '\n':
	last_column = 0;
	break;

    default:
	last_column++;
	break;
    }
    if ((last_column > 72) && (c != '\n'))
    {
	printf("\n     X");		/* all output happens here */
	last_column = 7;
    }
    putchar(c);				/* and here only */
}

void
outstr(s)				/* output string with line wrapping */
char *s;
{
    for ( ; *s ; ++s)
	output(*s);
}
